@@ -26,4 +26,6 @@ cp LICENSE tools/LICENSE.txt
 cp VERIFICATION tools/VERIFICATION.txt
 cp dist/act_windows_amd64/act.exe tools/
 choco pack act-cli.nuspec --version ${VERSION}
-choco push act-cli.${VERSION}.nupkg --api-key ${INPUT_APIKEY} -s https://push.chocolatey.org/ --timeout 180
+if [[ "$INPUT_PUSH" == "true" ]]; then
+  choco push act-cli.${VERSION}.nupkg --api-key ${INPUT_APIKEY} -s https://push.chocolatey.org/ --timeout 180
+fi
