@@ -38,6 +38,8 @@ type Input struct {
 	autoRemove            bool
 	artifactServerPath    string
 	artifactServerPort    string
+	jsonLogger            bool
+	noSkipCheckout        bool
 }
 
 func (i *Input) resolve(path string) string {
