@@ -17,6 +17,7 @@ import (
 	"github.com/go-git/go-billy/v5/helper/polyfill"
 	"github.com/go-git/go-billy/v5/osfs"
 	"github.com/go-git/go-git/v5/plumbing/format/gitignore"
+	"github.com/joho/godotenv"
 
 	"github.com/docker/cli/cli/connhelper"
 	"github.com/docker/docker/api/types"
@@ -52,6 +53,7 @@ type NewContainerInput struct {
 	Privileged  bool
 	UsernsMode  string
 	Platform    string
+	Hostname    string
 }
 
 // FileEntry is a file to copy to a container
@@ -69,10 +71,12 @@ type Container interface {
 	GetContainerArchive(ctx context.Context, srcPath string) (io.ReadCloser, error)
 	Pull(forcePull bool) common.Executor
 	Start(attach bool) common.Executor
-	Exec(command []string, env map[string]string, user string) common.Executor
+	Exec(command []string, env map[string]string, user, workdir string) common.Executor
 	UpdateFromEnv(srcPath string, env *map[string]string) common.Executor
+	UpdateFromImageEnv(env *map[string]string) common.Executor
 	UpdateFromPath(env *map[string]string) common.Executor
 	Remove() common.Executor
+	Close() common.Executor
 }
 
 // NewContainer creates a reference to a container
@@ -84,8 +88,7 @@ func NewContainer(input *NewContainerInput) Container {
 
 // supportsContainerImagePlatform returns true if the underlying Docker server
 // API version is 1.41 and beyond
-func supportsContainerImagePlatform(cli *client.Client) bool {
-	ctx := context.TODO()
+func supportsContainerImagePlatform(ctx context.Context, cli *client.Client) bool {
 	logger := common.Logger(ctx)
 	ver, err := cli.ServerVersion(ctx)
 	if err != nil {
@@ -103,7 +106,7 @@ func supportsContainerImagePlatform(cli *client.Client) bool {
 
 func (cr *containerReference) Create(capAdd []string, capDrop []string) common.Executor {
 	return common.
-		NewDebugExecutor("%sdocker create image=%s platform=%s entrypoint=%+q cmd=%+q", logPrefix, cr.input.Image, cr.input.Platform, cr.input.Entrypoint, cr.input.Cmd).
+		NewInfoExecutor("%sdocker create image=%s platform=%s entrypoint=%+q cmd=%+q", logPrefix, cr.input.Image, cr.input.Platform, cr.input.Entrypoint, cr.input.Cmd).
 		Then(
 			common.NewPipelineExecutor(
 				cr.connect(),
@@ -112,6 +115,7 @@ func (cr *containerReference) Create(capAdd []string, capDrop []string) common.E
 			).IfNot(common.Dryrun),
 		)
 }
+
 func (cr *containerReference) Start(attach bool) common.Executor {
 	return common.
 		NewInfoExecutor("%sdocker run image=%s platform=%s entrypoint=%+q cmd=%+q", logPrefix, cr.input.Image, cr.input.Platform, cr.input.Entrypoint, cr.input.Cmd).
@@ -125,14 +129,19 @@ func (cr *containerReference) Start(attach bool) common.Executor {
 			).IfNot(common.Dryrun),
 		)
 }
+
 func (cr *containerReference) Pull(forcePull bool) common.Executor {
-	return NewDockerPullExecutor(NewDockerPullExecutorInput{
-		Image:     cr.input.Image,
-		ForcePull: forcePull,
-		Platform:  cr.input.Platform,
-		Username:  cr.input.Username,
-		Password:  cr.input.Password,
-	})
+	return common.
+		NewInfoExecutor("%sdocker pull image=%s platform=%s username=%s forcePull=%t", logPrefix, cr.input.Image, cr.input.Platform, cr.input.Username, forcePull).
+		Then(
+			NewDockerPullExecutor(NewDockerPullExecutorInput{
+				Image:     cr.input.Image,
+				ForcePull: forcePull,
+				Platform:  cr.input.Platform,
+				Username:  cr.input.Username,
+				Password:  cr.input.Password,
+			}),
+		)
 }
 
 func (cr *containerReference) Copy(destPath string, files ...*FileEntry) common.Executor {
@@ -146,7 +155,7 @@ func (cr *containerReference) Copy(destPath string, files ...*FileEntry) common.
 func (cr *containerReference) CopyDir(destPath string, srcPath string, useGitIgnore bool) common.Executor {
 	return common.NewPipelineExecutor(
 		common.NewInfoExecutor("%sdocker cp src=%s dst=%s", logPrefix, srcPath, destPath),
-		cr.Exec([]string{"mkdir", "-p", destPath}, nil, ""),
+		cr.Exec([]string{"mkdir", "-p", destPath}, nil, "", ""),
 		cr.copyDir(destPath, srcPath, useGitIgnore),
 	).IfNot(common.Dryrun)
 }
@@ -160,16 +169,20 @@ func (cr *containerReference) UpdateFromEnv(srcPath string, env *map[string]stri
 	return cr.extractEnv(srcPath, env).IfNot(common.Dryrun)
 }
 
+func (cr *containerReference) UpdateFromImageEnv(env *map[string]string) common.Executor {
+	return cr.extractFromImageEnv(env).IfNot(common.Dryrun)
+}
+
 func (cr *containerReference) UpdateFromPath(env *map[string]string) common.Executor {
 	return cr.extractPath(env).IfNot(common.Dryrun)
 }
 
-func (cr *containerReference) Exec(command []string, env map[string]string, user string) common.Executor {
+func (cr *containerReference) Exec(command []string, env map[string]string, user, workdir string) common.Executor {
 	return common.NewPipelineExecutor(
-		common.NewInfoExecutor("%sdocker exec cmd=[%s] user=%s", logPrefix, strings.Join(command, " "), user),
+		common.NewInfoExecutor("%sdocker exec cmd=[%s] user=%s workdir=%s", logPrefix, strings.Join(command, " "), user, workdir),
 		cr.connect(),
 		cr.find(),
-		cr.exec(command, env, user),
+		cr.exec(command, env, user, workdir),
 	).IfNot(common.Dryrun)
 }
 
@@ -188,10 +201,7 @@ type containerReference struct {
 	input *NewContainerInput
 }
 
-func GetDockerClient(ctx context.Context) (*client.Client, error) {
-	var err error
-	var cli *client.Client
-
+func GetDockerClient(ctx context.Context) (cli *client.Client, err error) {
 	// TODO: this should maybe need to be a global option, not hidden in here?
 	//       though i'm not sure how that works out when there's another Executor :D
 	//		 I really would like something that works on OSX native for eg
@@ -219,6 +229,22 @@ func GetDockerClient(ctx context.Context) (*client.Client, error) {
 	return cli, err
 }
 
+func GetHostInfo(ctx context.Context) (info types.Info, err error) {
+	var cli *client.Client
+	cli, err = GetDockerClient(ctx)
+	if err != nil {
+		return info, err
+	}
+	defer cli.Close()
+
+	info, err = cli.Info(ctx)
+	if err != nil {
+		return info, err
+	}
+
+	return info, nil
+}
+
 func (cr *containerReference) connect() common.Executor {
 	return func(ctx context.Context) error {
 		if cr.cli != nil {
@@ -233,6 +259,16 @@ func (cr *containerReference) connect() common.Executor {
 	}
 }
 
+func (cr *containerReference) Close() common.Executor {
+	return func(ctx context.Context) error {
+		if cr.cli != nil {
+			cr.cli.Close()
+			cr.cli = nil
+		}
+		return nil
+	}
+}
+
 func (cr *containerReference) find() common.Executor {
 	return func(ctx context.Context) error {
 		if cr.id != "" {
@@ -266,7 +302,7 @@ func (cr *containerReference) remove() common.Executor {
 		}
 
 		logger := common.Logger(ctx)
-		err := cr.cli.ContainerRemove(context.Background(), cr.id, types.ContainerRemoveOptions{
+		err := cr.cli.ContainerRemove(ctx, cr.id, types.ContainerRemoveOptions{
 			RemoveVolumes: true,
 			Force:         true,
 		})
@@ -287,8 +323,8 @@ func (cr *containerReference) create(capAdd []string, capDrop []string) common.E
 		}
 		logger := common.Logger(ctx)
 		isTerminal := term.IsTerminal(int(os.Stdout.Fd()))
-
 		input := cr.input
+
 		config := &container.Config{
 			Image:      input.Image,
 			Cmd:        input.Cmd,
@@ -296,6 +332,7 @@ func (cr *containerReference) create(capAdd []string, capDrop []string) common.E
 			WorkingDir: input.WorkingDir,
 			Env:        input.Env,
 			Tty:        isTerminal,
+			Hostname:   input.Hostname,
 		}
 
 		mounts := make([]mount.Mount, 0)
@@ -308,7 +345,7 @@ func (cr *containerReference) create(capAdd []string, capDrop []string) common.E
 		}
 
 		var platSpecs *specs.Platform
-		if supportsContainerImagePlatform(cr.cli) && cr.input.Platform != "" {
+		if supportsContainerImagePlatform(ctx, cr.cli) && cr.input.Platform != "" {
 			desiredPlatform := strings.SplitN(cr.input.Platform, `/`, 2)
 
 			if len(desiredPlatform) != 2 {
@@ -344,7 +381,10 @@ var singleLineEnvPattern, mulitiLineEnvPattern *regexp.Regexp
 
 func (cr *containerReference) extractEnv(srcPath string, env *map[string]string) common.Executor {
 	if singleLineEnvPattern == nil {
-		singleLineEnvPattern = regexp.MustCompile("^([^=]+)=([^=]+)$")
+		// Single line pattern matches:
+		// SOME_VAR=data=moredata
+		// SOME_VAR=datamoredata
+		singleLineEnvPattern = regexp.MustCompile(`^([^=]*)\=(.*)$`)
 		mulitiLineEnvPattern = regexp.MustCompile(`^([^<]+)<<(\w+)$`)
 	}
 
@@ -389,6 +429,38 @@ func (cr *containerReference) extractEnv(srcPath string, env *map[string]string)
 	}
 }
 
+func (cr *containerReference) extractFromImageEnv(env *map[string]string) common.Executor {
+	envMap := *env
+	return func(ctx context.Context) error {
+		logger := common.Logger(ctx)
+
+		inspect, _, err := cr.cli.ImageInspectWithRaw(ctx, cr.input.Image)
+		if err != nil {
+			logger.Error(err)
+		}
+
+		imageEnv, err := godotenv.Unmarshal(strings.Join(inspect.Config.Env, "\n"))
+		if err != nil {
+			logger.Error(err)
+		}
+
+		for k, v := range imageEnv {
+			if k == "PATH" {
+				if envMap[k] == "" {
+					envMap[k] = v
+				} else {
+					envMap[k] += `:` + v
+				}
+			} else if envMap[k] == "" {
+				envMap[k] = v
+			}
+		}
+
+		env = &envMap
+		return nil
+	}
+}
+
 func (cr *containerReference) extractPath(env *map[string]string) common.Executor {
 	localEnv := *env
 	return func(ctx context.Context) error {
@@ -414,7 +486,7 @@ func (cr *containerReference) extractPath(env *map[string]string) common.Executo
 	}
 }
 
-func (cr *containerReference) exec(cmd []string, env map[string]string, user string) common.Executor {
+func (cr *containerReference) exec(cmd []string, env map[string]string, user, workdir string) common.Executor {
 	return func(ctx context.Context) error {
 		logger := common.Logger(ctx)
 		// Fix slashes when running on Windows
@@ -433,10 +505,22 @@ func (cr *containerReference) exec(cmd []string, env map[string]string, user str
 			envList = append(envList, fmt.Sprintf("%s=%s", k, v))
 		}
 
+		var wd string
+		if workdir != "" {
+			if strings.HasPrefix(workdir, "/") {
+				wd = workdir
+			} else {
+				wd = fmt.Sprintf("%s/%s", cr.input.WorkingDir, workdir)
+			}
+		} else {
+			wd = cr.input.WorkingDir
+		}
+		logger.Debugf("Working directory '%s'", wd)
+
 		idResp, err := cr.cli.ContainerExecCreate(ctx, cr.id, types.ExecConfig{
 			User:         user,
 			Cmd:          cmd,
-			WorkingDir:   cr.input.WorkingDir,
+			WorkingDir:   wd,
 			Env:          envList,
 			Tty:          isTerminal,
 			AttachStderr: true,
@@ -556,6 +640,11 @@ func (cr *containerReference) copyDir(dstPath string, srcPath string, useGitIgno
 				return err
 			}
 
+			// symlinks don't need to be copied
+			if fi.Mode()&os.ModeSymlink == os.ModeSymlink {
+				return nil
+			}
+
 			// open files for taring
 			f, err := os.Open(file)
 			if err != nil {
@@ -564,10 +653,6 @@ func (cr *containerReference) copyDir(dstPath string, srcPath string, useGitIgno
 
 			// copy file data into tar writer
 			if _, err := io.Copy(tw, f); err != nil {
-				if fi.Mode()&os.ModeSymlink == os.ModeSymlink {
-					// symlinks don't need to be copied, ignore this error
-					err = nil
-				}
 				return err
 			}
 
